% 1) e 2) Problema cinematico
% legge del moto z(t)=-(1/2*g*t^2)+v0*t+z0
% calcolo tempo finle di impatto al suolo:
% z(tf)=0   ->    0=gt^2/2+z0  ->  tf=sqrt(-2*z0/g)
clc, clear all
T=0.01;     % periodo di campionamento
z0=2;       % posizione iniziale della pallina
v0=0;       % velocit iniziale della pallina
g=9.81;     % accelerazione di gravit

ti=0;
tf=sqrt((2*z0)/g)
istanti=fix(tf/T);
tf=T*istanti;

t=ti:T:tf;
z=-0.5*g*(t.^2)+z0;

figure(1)
plot(t,z,'or')
title('Quota z in funzione del tempo t');
xlabel('Tempo (s)');
ylabel('Quota (m)');
grid


% 3) Errore stereo-fotogrammetrico: rumore a distribuzione normale
m=0;
sigma=0.005;
n=length(t);
noise=sigma*randn(1,n);

% 4) Quota affetta dall'errore stereo-fotogrammetrico
zr = z + noise;
hold on
plot(t,zr,'+k');
legend('Quota z non affetta da rumore','Quota z affetta da rumore');

% 5) Stima dell'accelerazione di gravita' a partire dal dato di posizione
%    (derivata seconda)
gz=(z(3:n)-2*z(2:n-1)+z(1:n-2))/T^2     %% posizione non affetta da rumore
gzr=(zr(3:n)-2*zr(2:n-1)+zr(1:n-2))/T^2; %% posizione affetta da rumore

% Notiamo che l'accelerazione di gravita' stimata a partire dal dato di
% posizione non affetto da rumore risulta essere molto precisa e pari a 
% 9.81 m/s^2

% L'accelerazione di gravita' stimata a partire dal dato di
% posizione affetto da rumore risulta invece perturbata da un errore elevato,
% infatti la derivata seconda amplifica il rumore presente nel dato di posizione

figure(2)
plot(t(2:n-1),gzr,'+r',t(2:n-1),gz,'+');
title('Stima puntuale dell''accelerazione g');
xlabel('Tempo (s)');
ylabel('Accelerazione di gravit (m/s^2)');
legend('Quota z affetta da rumore','Quota z non affetta da rumore')
grid

% 6) Metodo alternativo per la stima dell'accelerazione di gravit 
% a partire dal dato iniziale
% z=f(t^2)
% z(t)=g/2t^2+z0   ->  y=ax+b

x=t.^2;
y=z;                   % posizione non affetta da rumore
[p,s]=polyfit(x,y,1);  % approssimazione polinomiale 
                       % lineare metodo dei minimi quadrati
                       %  z=0.5*g*(t^2)+z0 ( y=p(1)*x+p(2) )
gp=2*p(1);
zp0=p(2);


xr=t.^2;
yr=zr;                      % posizione affetta da rumore
[pr,sr]=polyfit(xr,yr,1);   % approssimazione polinomiale 
                            % lineare metodo dei minimi quadrati
                            %  z=0.5*g*(t^2)+z0 ( y=p(1)*x+p(2) )
gpr=2*p(1);
zp0r=pr(2);
